@@ -69,6 +69,7 @@ class KateDocumentTest;
 
 class KateAutoIndent;
 class KateModOnHdPrompt;
+class KToggleAction;
 
 /**
  * @brief Backend of KTextEditor::Document related public KTextEditor interfaces.
@@ -267,6 +268,7 @@ public:
      * @return true on success
      */
     bool editInsertText(int line, int col, const QString &s);
+
     /**
      * Remove a string in the given line/column
      * @param line line number
@@ -279,7 +281,7 @@ public:
     /**
      * Mark @p line as @p autowrapped. This is necessary if static word warp is
      * enabled, because we have to know whether to insert a new line or add the
-     * wrapped words to the followin line.
+     * wrapped words to the following line.
      * @param line line number
      * @param autowrapped autowrapped?
      * @return true on success
@@ -297,6 +299,7 @@ public:
      * @return true on success
      */
     bool editWrapLine(int line, int col, bool newLine = true, bool *newLineAdded = nullptr);
+
     /**
      * Unwrap @p line. If @p removeLine is true, we force to join the lines. If
      * @p removeLine is true, @p length is ignored (eg not needed).
@@ -313,6 +316,7 @@ public:
      * @return true on success
      */
     bool editInsertLine(int line, const QString &s);
+
     /**
      * Remove a line
      * @param line line number
@@ -323,17 +327,30 @@ public:
     bool editRemoveLines(int from, int to);
 
     /**
-     * Remove a line
+     * Warp a line
      * @param startLine line to begin wrapping
      * @param endLine line to stop wrapping
      * @return true on success
      */
     bool wrapText(int startLine, int endLine);
+
+    /**
+     * Wrap lines touched by the selection with respect of existing paragraphs.
+     * To do so will the paragraph prior to the wrap joined as one single line
+     * which cause an almost perfect wrapped paragraph as long as there are no
+     * unneeded spaces exist or some formatting like this comment block.
+     * Without any selection the current line is wrapped.
+     * Empty lines around each paragraph are untouched.
+     * @param first line to begin wrapping
+     * @param last line to stop wrapping
+     * @return true on success
+     */
+    bool wrapParagraph(int first, int last);
 //END LINE BASED INSERT/REMOVE STUFF
 
 Q_SIGNALS:
     /**
-     * Emmitted when text from @p line was wrapped at position pos onto line @p nextLine.
+     * Emitted when text from @p line was wrapped at position pos onto line @p nextLine.
      */
     void editLineWrapped(int line, int col, int len);
 
@@ -360,9 +377,9 @@ private:
     QStack<int> editStateStack;
     bool editIsRunning = false;
     bool m_undoMergeAllEdits = false;
+    KTextEditor::Cursor m_editLastChangeStartCursor = KTextEditor::Cursor::invalid();
     QStack<QSharedPointer<KTextEditor::MovingCursor>> m_editingStack;
     int m_editingStackPosition = -1;
-    static const int s_editingStackSizeLimit = 32;
 
     //
     // KTextEditor::UndoInterface stuff
@@ -381,7 +398,7 @@ public Q_SLOTS:
      * If the consecutive editings happens in the same line, then remove
      * the previous and add the new one with updated column no.
      */
-    void saveEditingPositions(KTextEditor::Document *, const KTextEditor::Range &range);
+    void saveEditingPositions(const KTextEditor::Cursor &cursor);
 
 public:
     uint undoCount() const;
@@ -744,6 +761,13 @@ public:
 
     void setModified(bool m) override;
 
+    bool isAutoReload();
+    KToggleAction* autoReloadToggleAction() { return m_autoReloadMode; };
+    void delayAutoReload();
+
+private Q_SLOTS:
+    void autoReloadToggled(bool b);
+
 private:
     void activateDirWatch(const QString &useFileName = QString());
     void deactivateDirWatch();
@@ -966,6 +990,7 @@ private:
 private Q_SLOTS:
     void onModOnHdSaveAs();
     void onModOnHdReload();
+    void onModOnHdAutoReload();
     void onModOnHdIgnore();
 
 public:
@@ -1011,7 +1036,7 @@ private:
      * Create a git compatible sha1 checksum of the file, if it is a local file.
      * The result can be accessed through KateBuffer::digest().
      *
-     * @return wheather the operation was attempted and succeeded.
+     * @return whether the operation was attempted and succeeded.
      */
     bool createDigest();
 
@@ -1082,6 +1107,8 @@ private:
     bool m_userSetEncodingForNextReload = false;
 
     bool m_modOnHd = false;
+    KToggleAction* m_autoReloadMode;
+    QTimer m_autoReloadThrottle;
     ModifiedOnDiskReason m_modOnHdReason = OnDiskUnmodified;
     ModifiedOnDiskReason m_prevModOnHdReason = OnDiskUnmodified;
 
@@ -1164,7 +1191,7 @@ private:
     static bool checkIntValue(QString value, int *result);
     /**
       Feeds value into @p col using QColor::setNamedColor() and returns
-      wheather the color is valid
+      whether the color is valid
     */
     static bool checkColorValue(QString value, QColor &col);
 
@@ -1218,8 +1245,8 @@ public:
 
 public Q_SLOTS:
     void clearDictionaryRanges();
+    void setDictionary(const QString &dict, const KTextEditor::Range &range, bool blockmode);
     void setDictionary(const QString &dict, const KTextEditor::Range &range);
-    void revertToDefaultDictionary(const KTextEditor::Range &range);
     void setDefaultDictionary(const QString &dict);
     void onTheFlySpellCheckingEnabled(bool enable);
     void refreshOnTheFlyCheck(const KTextEditor::Range &range = KTextEditor::Range::invalid());
