@@ -33,12 +33,12 @@
 
 #include <KLocalizedString>
 
-#include <QRegExp>
+#include <QRegularExpression>
 
 //BEGIN CoreCommands
 KateCommands::CoreCommands *KateCommands::CoreCommands::m_instance = nullptr;
 
-// this returns wheather the string s could be converted to
+// this returns whether the string s could be converted to
 // a bool value, one of on|off|1|0|true|false. the argument val is
 // set to the extracted value in case of success
 static bool getBoolArg(const QString &t, bool *val)
@@ -204,7 +204,7 @@ bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
     }
 
     //create a list of args
-    QStringList args(_cmd.split(QRegExp(QLatin1String("\\s+")), QString::SkipEmptyParts));
+    QStringList args(_cmd.split(QRegularExpression(QLatin1String("\\s+")), QString::SkipEmptyParts));
     QString cmd(args.takeFirst());
 
     // ALL commands that takes no arguments.
@@ -352,7 +352,7 @@ bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
             v->doc()->setWordWrapAt(val);
         } else if (cmd == QLatin1String("goto")) {
             if (args.first().at(0) == QLatin1Char('-') || args.first().at(0) == QLatin1Char('+')) {
-                // if the number starts with a minus or plus sign, add/subract the number
+                // if the number starts with a minus or plus sign, add/subtract the number
                 val = v->cursorPosition().line() + val;
             } else {
                 val--; // convert given line number to the internal representation of line numbers
@@ -403,7 +403,7 @@ bool KateCommands::CoreCommands::exec(KTextEditor::View *view,
             } else if (cmd == QLatin1String("set-show-tabs")) {
                 config->setShowTabs(enable);
             } else if (cmd == QLatin1String("set-show-trailing-spaces")) {
-                config->setShowSpaces(enable);
+                config->setShowSpaces(enable ? KateDocumentConfig::Trailing : KateDocumentConfig::None);
             } else if (cmd == QLatin1String("set-word-wrap")) {
                 v->doc()->setWordWrap(enable);
             }
@@ -449,8 +449,8 @@ KCompletion *KateCommands::CoreCommands::completionObject(KTextEditor::View *vie
 
     if (cmd == QLatin1String("set-highlight")) {
         QStringList l;
-        for (int i = 0; i < KateHlManager::self()->highlights(); i++) {
-            l << KateHlManager::self()->hlName(i);
+        for (const auto &hl : KateHlManager::self()->modeList()) {
+            l << hl.name();
         }
 
         KateCmdShellCompletion *co = new KateCmdShellCompletion();
@@ -499,19 +499,23 @@ bool KateCommands::Character::exec(KTextEditor::View *view, const QString &_cmd,
     QString cmd = _cmd;
 
     // hex, octal, base 9+1
-    QRegExp num(QLatin1String("^char *(0?x[0-9A-Fa-f]{1,4}|0[0-7]{1,6}|[0-9]{1,5})$"));
-    if (num.indexIn(cmd) == -1) {
+    QRegularExpression num(QLatin1String("^char *(0?x[0-9A-Fa-f]{1,4}|0[0-7]{1,6}|[0-9]{1,5})$"));
+    QRegularExpressionMatch match = num.match(cmd);
+    if (!match.hasMatch()) {
         return false;
     }
 
-    cmd = num.cap(1);
+    cmd = match.captured(1);
 
     // identify the base
 
     unsigned short int number = 0;
     int base = 10;
-    if (cmd[0] == QLatin1Char('x') || cmd.startsWith(QLatin1String("0x"))) {
-        cmd.remove(QRegExp(QLatin1String("^0?x")));
+    if (cmd.startsWith(QLatin1Char('x'))) {
+        cmd.remove(0, 1);
+        base = 16;
+    } else if (cmd.startsWith(QStringLiteral("0x"))) {
+        cmd.remove(0, 2);
         base = 16;
     } else if (cmd[0] == QLatin1Char('0')) {
         base = 8;
