@@ -31,6 +31,7 @@
 #include <KLocalizedString>
 #include <KIconUtils>
 #include <KAcceleratorManager>
+#include <Sonnet/Speller>
 
 #include <QHBoxLayout>
 #include <QInputDialog>
@@ -112,10 +113,35 @@ KateStatusBar::KateStatusBar(KTextEditor::ViewPrivate *view)
     m_inputMode->setWhatsThis(i18n("Insert mode and VI input mode indicator. Click to change the mode."));
     connect(m_inputMode, &StatusBarButton::clicked, [=] { m_view->currentInputMode()->toggleInsert(); });
 
+    /**
+     * Add dictionary button which allows user to switch dictionary of the document
+     */
+    m_dictionary = new StatusBarButton(this);
+    topLayout->addWidget(m_dictionary, 0);
+    m_dictionary->setWhatsThis(i18n("Change dictionary"));
+    m_dictionaryMenu = new KateStatusBarOpenUpMenu(m_dictionary);
+    m_dictionaryMenu->addAction(m_view->action("tools_change_dictionary"));
+    m_dictionaryMenu->addAction(m_view->action("tools_clear_dictionary_ranges"));
+    m_dictionaryMenu->addAction(m_view->action("tools_toggle_automatic_spell_checking"));
+    m_dictionaryMenu->addAction(m_view->action("tools_spelling_from_cursor"));
+    m_dictionaryMenu->addAction(m_view->action("tools_spelling"));
+    m_dictionaryMenu->addSeparator();
+    m_dictionaryGroup = new QActionGroup(m_dictionaryMenu);
+    QMapIterator<QString, QString> i(Sonnet::Speller().preferredDictionaries());
+    while (i.hasNext()) {
+        i.next();
+        QAction *action = m_dictionaryGroup->addAction(i.key());
+        action->setData(i.value());
+        action->setToolTip(i.key());
+        action->setCheckable(true);
+        m_dictionaryMenu->addAction(action);
+    }
+    m_dictionary->setMenu(m_dictionaryMenu);
+    connect(m_dictionaryGroup, &QActionGroup::triggered, this, &KateStatusBar::changeDictionary);
+
     /**
      * allow to change indentation configuration
      */
-
     m_tabsIndent = new StatusBarButton(this);
     topLayout->addWidget(m_tabsIndent);
 
@@ -175,9 +201,13 @@ KateStatusBar::KateStatusBar(KTextEditor::ViewPrivate *view)
     connect(m_view, &KTextEditor::View::selectionChanged, this, &KateStatusBar::selectionChanged);
     connect(m_view->document(), &KTextEditor::DocumentPrivate::modifiedChanged, this, &KateStatusBar::modifiedChanged);
     connect(m_view->doc(), &KTextEditor::DocumentPrivate::modifiedOnDisk, this, &KateStatusBar::modifiedChanged);
+    connect(m_view->doc(), &KTextEditor::DocumentPrivate::readWriteChanged, this, &KateStatusBar::modifiedChanged);
     connect(m_view->doc(), &KTextEditor::DocumentPrivate::configChanged, this, &KateStatusBar::documentConfigChanged);
     connect(m_view->document(), &KTextEditor::DocumentPrivate::modeChanged, this, &KateStatusBar::modeChanged);
     connect(m_view, &KTextEditor::ViewPrivate::configChanged, this, &KateStatusBar::configChanged);
+    connect(m_view->doc(), &KTextEditor::DocumentPrivate::defaultDictionaryChanged, this, &KateStatusBar::updateDictionary);
+    connect(m_view->doc(), &KTextEditor::DocumentPrivate::dictionaryRangesPresent, this, &KateStatusBar::updateDictionary);
+    connect(m_view, &KTextEditor::ViewPrivate::caretChangedRange, this, &KateStatusBar::updateDictionary);
 
     updateStatus();
     toggleWordCount(KateViewConfig::global()->showWordCount());
@@ -220,6 +250,7 @@ void KateStatusBar::updateStatus()
     modifiedChanged();
     documentConfigChanged();
     modeChanged();
+    updateDictionary();
 }
 
 void KateStatusBar::selectionChanged()
@@ -248,8 +279,8 @@ void KateStatusBar::cursorPositionChanged()
 {
     KTextEditor::Cursor position(m_view->cursorPositionVirtual());
 
+    // Update line/column label
     QString text;
-
     if (KateViewConfig::global()->showLineCount()) {
         text = i18n("Line %1 of %2, Column %3", QLocale().toString(position.line() + 1)
                                               , QLocale().toString(m_view->doc()->lines())
@@ -260,47 +291,101 @@ void KateStatusBar::cursorPositionChanged()
                                         , QLocale().toString(position.column() + 1)
                    );
     }
-
     if (m_wordCounter) {
         text.append(QStringLiteral(", ") + m_wordCount);
     }
-
     m_cursorPosition->setText(text);
 }
 
+void KateStatusBar::updateDictionary()
+{
+    QString newDict;
+    // Check if at the current cursor position is a special dictionary in use
+    KTextEditor::Cursor position(m_view->cursorPositionVirtual());
+    const QList<QPair<KTextEditor::MovingRange *, QString>> dictRanges = m_view->doc()->dictionaryRanges();
+    for (auto rangeDictPair : dictRanges) {
+        const KTextEditor::MovingRange *range = rangeDictPair.first;
+        if (range->contains(position) || range->end() == position) {
+            newDict = rangeDictPair.second;
+            break;
+        }
+    }
+    // Check if the default dictionary is in use
+    if (newDict.isEmpty()) {
+        newDict = m_view->doc()->defaultDictionary();
+        if (newDict.isEmpty()) {
+            newDict = Sonnet::Speller().defaultLanguage();
+        }
+    }
+    // Update button and menu only on a changed dictionary
+    if (!m_dictionaryGroup->checkedAction() || (m_dictionaryGroup->checkedAction()->data().toString() != newDict) || m_dictionary->text().isEmpty()) {
+        bool found = false;
+        // Remove "-w_accents -variant_0" and such from dict-code to keep it small and clean
+        m_dictionary->setText(newDict.section(QLatin1Char('-'), 0, 0));
+        // For maximum user clearness, change the checked menu option
+        m_dictionaryGroup->blockSignals(true);
+        for (auto a : m_dictionaryGroup->actions()) {
+            if (a->data().toString() == newDict) {
+                a->setChecked(true);
+                found = true;
+                break;
+            }
+        }
+        if (!found) {
+            // User has chose some other dictionary from combo box, we need to add that
+            QString dictName = Sonnet::Speller().availableDictionaries().key(newDict);
+            QAction *action = m_dictionaryGroup->addAction(dictName);
+            action->setData(newDict);
+            action->setCheckable(true);
+            action->setChecked(true);
+            m_dictionaryMenu->addAction(action);
+        }
+        m_dictionaryGroup->blockSignals(false);
+    }
+}
+
 void KateStatusBar::modifiedChanged()
 {
     const bool mod = m_view->doc()->isModified();
     const bool modOnHD = m_view->doc()->isModifiedOnDisc();
+    const bool readOnly = !m_view->doc()->isReadWrite();
 
     /**
      * combine to modified status, update only if changed
      */
-    unsigned int newStatus = (unsigned int)mod | ((unsigned int)modOnHD << 1);
+    unsigned int newStatus = (unsigned int)mod | ((unsigned int)modOnHD << 1) | ((unsigned int)readOnly << 2);
     if (m_modifiedStatus == newStatus)
         return;
 
     m_modifiedStatus = newStatus;
     switch (m_modifiedStatus) {
+    case 0x0:
+        m_modified->setIcon(QIcon::fromTheme(QStringLiteral("text-plain")));
+        m_modified->setWhatsThis(i18n("Meaning of current icon: Document was not modified since it was loaded"));
+        break;
+
     case 0x1:
+    case 0x5:
         m_modified->setIcon(QIcon::fromTheme(QStringLiteral("document-save")));
         m_modified->setWhatsThis(i18n("Meaning of current icon: Document was modified since it was loaded"));
         break;
 
     case 0x2:
+    case 0x6:
         m_modified->setIcon(QIcon::fromTheme(QStringLiteral("dialog-warning")));
         m_modified->setWhatsThis(i18n("Meaning of current icon: Document was modified or deleted by another program"));
         break;
 
     case 0x3:
+    case 0x7:
         m_modified->setIcon(QIcon(KIconUtils::addOverlay(QIcon::fromTheme(QStringLiteral("document-save")),
                                        QIcon(QStringLiteral("emblem-important")), Qt::TopLeftCorner)));
         m_modified->setWhatsThis(QString());
         break;
 
     default:
-        m_modified->setIcon(QIcon::fromTheme(QStringLiteral("text-plain")));
-        m_modified->setWhatsThis(i18n("Meaning of current icon: Document was not modified since it was loaded"));
+        m_modified->setIcon(QIcon::fromTheme(QStringLiteral("lock")));
+        m_modified->setWhatsThis(i18n("Meaning of current icon: Document is in read-only mode"));
         break;
     }
 }
@@ -452,7 +537,11 @@ void KateStatusBar::toggleWordCount(bool on)
 void KateStatusBar::wordCountChanged(int wordsInDocument, int wordsInSelection, int charsInDocument, int charsInSelection)
 {
     if (m_wordCounter) {
-        m_wordCount = i18n("Words %1/%2, Chars %3/%4", wordsInDocument, wordsInSelection, charsInDocument, charsInSelection);
+        m_wordCount = i18nc(
+            "%1 and %3 are the selected words/chars count, %2 and %4 are the total words/chars count.",
+            "Words %1/%2, Chars %3/%4",
+            wordsInSelection, wordsInDocument, charsInSelection, charsInDocument
+        );
     } else {
         m_wordCount.clear();
     }
@@ -464,3 +553,16 @@ void KateStatusBar::configChanged()
 {
     toggleWordCount(m_view->config()->showWordCount());
 }
+
+void KateStatusBar::changeDictionary(QAction *action)
+{
+    const QString dictionary = action->data().toString();
+    m_dictionary->setText(dictionary);
+    // Code stolen from KateDictionaryBar::dictionaryChanged
+    KTextEditor::Range selection = m_view->selectionRange();
+    if (selection.isValid() && !selection.isEmpty()) {
+        m_view->doc()->setDictionary(dictionary, selection);
+    } else {
+        m_view->doc()->setDefaultDictionary(dictionary);
+    }
+}
